import { Workflow } from "../workflow-types";

export const TEMPLATE_OWNER: Workflow["owner"] = {
  id: "team-templates",
  name: "Orcheo Templates",
  avatar: "https://avatar.vercel.sh/templates",
};
